close all
clear all

% select a particular quadrangle
x=zeros(1,8);
y=zeros(1,8);
x(3)=1.5;
x(5)=2;y(5)=2;
y(7)=1;

% set mid-points
x(2:2:6)=(x(1:2:5)+x(3:2:7))/2;
y(2:2:6)=(y(1:2:5)+y(3:2:7))/2;
x(8)=(x(7)+x(1))/2;
y(8)=(y(7)+y(1))/2;


% Attempt to map (xi,eta)=M*(x,y,xy,x^2,y^2,x^2y,xy^2,1)
% M of size [2,8] numbered as 
%  M(1) M(3) M(5)...
%  M(2) M(4) M(6)...       
%
% K*M=S
K=[x(:),zeros(8,1), y(:),zeros(8,1),(x(:).*y(:)),zeros(8,1),x(:).^2,zeros(8,1),...
    y(:).^2,zeros(8,1),(x(:).^2).*y(:),zeros(8,1),x(:).*(y(:).^2),zeros(8,1),ones(8,1),zeros(8,1);...
    zeros(8,1),x(:),zeros(8,1), y(:),zeros(8,1),(x(:).*y(:)),zeros(8,1),x(:).^2,zeros(8,1),...
    y(:).^2,zeros(8,1),(x(:).^2).*y(:),zeros(8,1),x(:).*(y(:).^2),zeros(8,1),ones(8,1)];
S=[-1;0;1;1;1;0;-1;-1;  -1;-1;-1;0;1;1;1;0];
M=K\S;
M=reshape(M,[2,8]);

% Now for any point of global system coordinate (xx,yy), we have an
% approximate local (reference) coordinate (xi,eta) such that
% [xi;eta]=M*[xx;yy;xx*yy;xx^2;yy^2;(xx^2)*yy;xx*(yy^2);1];


% Validation from exhaustive sampling
xa=[0.01:0.02:1.99];
ya=[0.01:0.02:1.99];
[Ya,Xa]=meshgrid(ya,xa);
deltaX=NaN(size(Xa));
deltaY=NaN(size(Xa));
deltaX2=NaN(size(Xa));
deltaY2=NaN(size(Xa));
inside=inpolygon(Ya(:),Xa(:),y,x);
ind=find(inside==1);
for ii=1:length(ind)
    xx=Xa(ind(ii));yy=Ya(ind(ii));
    xi=M(1,:)*[xx;yy;xx*yy;xx^2;yy^2;(xx^2)*yy;xx*(yy^2);1];
    eta=M(2,:)*[xx;yy;xx*yy;xx^2;yy^2;(xx^2)*yy;xx*(yy^2);1];
    psi=Q8(xi,eta);
    X2=psi*[x(1);x(2);x(3);x(4);x(5);x(6);x(7);x(8)];
    Y2=psi*[y(1);y(2);y(3);y(4);y(5);y(6);y(7);y(8)];
    deltaX(ind(ii))=X2-xx;
    deltaY(ind(ii))=Y2-yy;
    deltaX2(ind(ii))=deltaX(ind(ii));
    deltaY2(ind(ii))=deltaY(ind(ii));
    while (deltaX2(ind(ii))^2+deltaY2(ind(ii))^2>1.e-10)
        [dpsidx,dpsidy]=gradQ8(xi,eta);
        Ma(1,1)=dpsidx*[x(1);x(2);x(3);x(4);x(5);x(6);x(7);x(8)];    % dXdxi
        Ma(2,1)=dpsidx*[y(1);y(2);y(3);y(4);y(5);y(6);y(7);y(8)];    % dYdxi
        Ma(1,2)=dpsidy*[x(1);x(2);x(3);x(4);x(5);x(6);x(7);x(8)];    % dXdeta
        Ma(2,2)=dpsidy*[y(1);y(2);y(3);y(4);y(5);y(6);y(7);y(8)];    % dYdeta
        Ma=Ma+1.e-6*eye(2);
        Na=inv(Ma);
        D=Na*[deltaX2(ind(ii));deltaY2(ind(ii))];
        xi=xi-D(1);
        eta=eta-D(2);
        psi=Q8(xi,eta);
        X3=psi*[x(1);x(2);x(3);x(4);x(5);x(6);x(7);x(8)];
        Y3=psi*[y(1);y(2);y(3);y(4);y(5);y(6);y(7);y(8)];
        deltaX2(ind(ii))=X3-xx;
        deltaY2(ind(ii))=Y3-yy;
    end
end

figure
colormap(hot)
imagesc(ya,xa,deltaX,.01*[-1 1])
axis xy
colorbar
title('Error in x')

figure
colormap(hot)
imagesc(ya,xa,deltaY,.01*[-1 1])
axis xy
colorbar
title('Error in y')

figure
colormap(hot)
imagesc(ya,xa,deltaX2,.001*[-1 1])
axis xy
colorbar
title('Error 2 in x')

figure
colormap(hot)
imagesc(ya,xa,deltaY2,.001*[-1 1])
axis xy
colorbar
title('Error 2 in y')
