function [U,Resid,Param]=Compute_core(FlagBatch,FlagVisu,WorkFile,NewJob,im1,imdef,U,Mtot,Mregul,Param)

dU=nan(size(U));

% Mask
load(NewJob.Mesh);
load(WorkFile.ImMesh,'RefMesh','T','W1','W2','W3');
if NewJob.UseMask
    load(WorkFile.ImMask,'Mask');
else
    Mask=logical(ones(size(im1)));
end
Mask=logical(T>0)&Mask;

% Image characteristics
image.dyn  = max(im1(Mask))-min(im1(Mask));
image.mean = mean(im1(Mask));
image.std  = std(im1(Mask));


ROI=NewJob.ROI;
coorx=[ROI(1),ROI(3)];
coory=[ROI(2),ROI(4)];
[Yref,Xref]=meshgrid([ROI(2):ROI(4)],[ROI(1):ROI(3)]);
nxdef=size(imdef,1);
nydef=size(imdef,2);
[Ydef,Xdef]=meshgrid([1:nydef],[1:nxdef]);
jm1=imdef(ROI(1):ROI(3),ROI(2):ROI(4));
Resid=zeros(size(im1));
Resid(Mask)=im1(Mask)-jm1(Mask);
resnorm=std(Resid(:))/image.dyn;
disp(sprintf('  Initial Residual = %g',resnorm));

%% =============== PG CORE ============================
if FlagVisu
    FigUx  =figure('Position',[    1   1   600   500]);
    FigUy  =figure('Position',[  601   1   600   500]);
    FigDiff=figure('Position',[  401 200   600   500]);
end

%% %%%%%%%%%%%%%%% Computation loop %%%%%%%%%%%%%%%%%%%

for iter=1:Param.Itermax

    Uxpix=InterpMeshNode(WorkFile,NewJob,U(1:2:end-1));
    Uypix=InterpMeshNode(WorkFile,NewJob,U(2:2:end));
    ind=find(isnan(Uxpix));

%% %%%%%%%%%%%%%%% Correct image %%%%%%%%%%%%%%%%%%%%%%
    Xcor=Xref+Uxpix;
    Ycor=Yref+Uypix;
    jm1=zeros(size(im1));
    jm1(Mask)=interp2(Ydef,Xdef,imdef,Ycor(Mask),Xcor(Mask),Param.InterpMethod);
    jm1=jm1/std(jm1(Mask))*image.std;
    jm1=jm1-mean(jm1(Mask))+image.mean;

    if FlagVisu
        FigUx=ShowIm(Uxpix,'$U_x$','ROI',ROI,'Handle',FigUx,'Colormap','hot','Position',[.14 .14 .7 .73]);
        colorbar
        FigUy=ShowIm(Uypix,'$U_y$','ROI',ROI,'Handle',FigUy,'Colormap','hot','Position',[.14 .14 .7 .73]);
        colorbar
        FigDiff=ShowIm(abs(im1-jm1).*Mask,'Residual','ROI',ROI,'Handle',FigDiff,'Colormap','hot','Position',[.14 .14 .7 .73]);
        colorbar
    end

%% %%%%%%%%%%%%%%% Get residuals %%%%%%%%%%%%%%%%%%%%%%
    Resid=im1-jm1;
    oldresnorm=resnorm;
    resnorm=std(Resid(Mask))/image.dyn;
    disp(sprintf('     Iter        = %g',iter));
    disp(sprintf('        Residual = %g',resnorm));
    disp(sprintf('        dUx      = %g',mean(dU(1:2:end))));
    disp(sprintf('        dUy      = %g',mean(dU(2:2:end))));
    disp(sprintf('          Ux     = %g',mean(U(1:2:end))));
    disp(sprintf('          Uy     = %g',mean(U(2:2:end))));

%% %%%%%%%%%%%%%%% Compute second member %%%%%%%%%%%%%%%%%%%%%%

    S=TriDICSecond2(WorkFile,NewJob,Resid);
    dU=Mtot\(S-Mregul*U);
    U=U+dU;
%    if (norm([mean(dU(1:2:end)),mean(dU(2:2:end))])<Param.CvCriter|(resnorm>oldresnorm))&(iter>=Param.Itermin)
    if (norm([mean(dU(1:2:end)),mean(dU(2:2:end))])<Param.CvCriter)&(iter>=Param.Itermin)
        Param.Iterfin = iter;
        break
    end

end
Param.Iterfin = iter;