function  [TRI,Z,NNodeIn,NNodeEdge]=RefineMesh(d,Z,NNodeIn,NNodeEdge,Region,FlagPlot)

% given a polygonal domain of vertices za (in the complex plane),
% a T3 mesh of approximative element size d  is produced.  
% nodes are given as a vector of complex coordinates Z
% Nptin is the number of inner points Z(1:Nptin)
% Nptedge is the number of points on the boundary Z(Nptin+1:end)
% TRI gives the element list (triplets of node numbers)
% iff FlagPlot=1 figures are shown

if Region(end)~=Region(1)
    Region=[Region(:);Region(1)];
end
Xreg=real(Region);
Yreg=imag(Region);
Zedge=Z(NNodeIn+1:end);
Zin=Z(1:NNodeIn);
inside=inpolygon(imag(Zin(:)),real(Zin(:)),Yreg,Xreg);
ind=find(~inside);
Zin=Zin(ind);
x0=min(Xreg);x1=max(Xreg);
y0=min(Yreg);y1=max(Yreg);
randn('state',sum(100*clock))
x=[x0:d:x1];y=[y0:d:y1];
[Y,X]=meshgrid(y,x);
Y(1:2:end,:)=Y(1:2:end,:)+d/2;
% X(:)=X(:)+randn(length(X(:)),1)*d/10;
% Y(:)=Y(:)+randn(length(Y(:)),1)*d/10;
Zcenter=mean(Region);
IN = inpolygon(X,Y,real(Zcenter+.95*(Region-Zcenter)),imag(Zcenter+.95*(Region-Zcenter)));
Zin2=X+i*Y;
ind=find(IN==1);
Zin2=Zin2(ind);
NNodeIn=length(Zin)+length(Zin2);
Z=[Zin;Zin2;Zedge];
TRI=delaunay(real(Z),imag(Z));
NNodeTot=length(Z);

for iter=1:10
    for jter=1:10
        Fz=zeros(NNodeIn,1);
        for ii=1:size(TRI,1)
            ind=TRI(ii,:);
            for jj=1:3
                if ind(jj)<=NNodeIn
                    ki=ind(jj);
                    kj=ind(mod(jj,3)+1);
                    kk=ind(mod(jj+1,3)+1);
                    Fz(ki)=Fz(ki)+(Z(kj)-Z(ki))+(Z(kk)-Z(ki));
                end
            end
        end
        Z(1:NNodeIn)=Z(1:NNodeIn)+Fz(1:NNodeIn)/10;
    end
    TRI=delaunay(real(Z),imag(Z));
end

% Loop for non convex boundaries
for ii=1:size(TRI,1)
    iinode=TRI(ii,:);
    Zcenter=mean(Z(iinode(:)));
    valid(ii)=inpolygon(real(Zcenter),imag(Zcenter),real(Zedge),imag(Zedge));
end
ind=find(valid);
TRI=TRI(ind,:);

if FlagPlot
    figure
    triplot(TRI,imag(Z),real(Z),'k')
    axis ij;axis image
    xlabel('y')
    ylabel('x')
    title('Mesh')
    drawnow
end
