function  [Z]=SamplePolyBoundary(d,za,FlagPlot)

% given a polygonal domain of vertices za (in the complex plane),
% the boundary is sampled by steps of approximative size  d 
% to produce a series of points Z (in the complex plane) 
% discretizing the boundary
% if FlagPlot=1 figures are shown

% Close polygon if needed
if abs(za(end)-za(1))>.1
    za=[za(:);za(1)];
end

nvertex=length(za);
%
Z(1)=za(1);
for ii=1:nvertex-1;
    z0=za(ii);z1=za(ii+1);
    ds=abs(z1-z0);
    n=round(ds/d);
    n=max(n,1);
    t=[1:n]'/n;
    Z=[Z;z0+t*(z1-z0)];
end
Z=Z(1:end-1);


if FlagPlot
    xmin=min(real(za));
    xmax=max(real(za));
    dx=xmax-xmin;
    xmin=xmin-0.2*dx;
    xmax=xmax+0.2*dx;
    ymin=min(imag(za));
    ymax=max(imag(za));
    dy=ymax-ymin;
    ymin=ymin-0.2*dy;
    ymax=ymax+0.2*dy;

    figure
    line(imag(za),real(za),'Color','r','LineWidth',2)
    hold on
    line(imag(Z),real(Z),'Color','k','Marker','.')
    axis ij
    axis image
    xlim([ymin ymax])
    ylim([xmin xmax])
    xlabel('$y$','Interpreter','latex')
    ylabel('$x$','Interpreter','latex')
    title('SAMPLE POLY BOUNDARY')
    pause 
    close
end

