function  [TRI,Z,NNodeIn,NNodeEdge]=SimpleMesh(d,za,FlagPlot)

% given a polygonal domain of vertices za (in the complex plane),
% a T3 mesh of approximative element size d is produced.  
% nodes are given as a vector of complex coordinates Z
% NNodeIn is the number of inner nodes Z(1:NNodeIn)
% NNodeEdge is the number of nodes on the boundary Z(NNodeIn+1:end)
% TRI gives the element list (triplets of node numbers)
% iff FlagPlot=1 figures are shown

randn('state',sum(100*clock))
x0=min(real(za));x1=max(real(za));
y0=min(imag(za));y1=max(imag(za));
x=[x0:d:x1];y=[y0:d:y1];
[Y,X]=meshgrid(y,x);
Y(1:2:end,:)=Y(1:2:end,:)+d/2;
% X(:)=X(:)+randn(length(X(:)),1)*d/10;
% Y(:)=Y(:)+randn(length(Y(:)),1)*d/10;
Zcenter=mean(za);
IN = inpolygon(X,Y,real(Zcenter+.98*(za-Zcenter)),imag(Zcenter+.98*(za-Zcenter)));
Z=X+i*Y;
ind=find(IN==1);
Z=Z(ind);
NNodeIn=length(Z);
Zedge=SamplePolyBoundary(d,za,0).';
Zedge=Zedge(1:end-1);
Z=[Z;Zedge];
NNodeEdge=length(Zedge);
TRI=delaunay(real(Z),imag(Z));
Npttot=length(Z);

for iter=1:10
    for jter=1:10
        Fz=zeros(NNodeIn,1);
        for ii=1:size(TRI,1)
            ind=TRI(ii,:);
            for jj=1:3
                if ind(jj)<=NNodeIn
                    ki=ind(jj);
                    kj=ind(mod(jj,3)+1);
                    kk=ind(mod(jj+1,3)+1);
                    Fz(ki)=Fz(ki)+(Z(kj)-Z(ki))+(Z(kk)-Z(ki));
                end
            end
        end
        Z(1:NNodeIn)=Z(1:NNodeIn)+Fz(1:NNodeIn)/10;
    end
    TRI=delaunay(real(Z),imag(Z));
end

% Loop for non convex boundaries
for ii=1:size(TRI,1)
    iinode=TRI(ii,:);
    Zcenter=mean(Z(iinode(:)));
    valid(ii)=inpolygon(real(Zcenter),imag(Zcenter),real(za),imag(za));
end
ind=find(valid);
TRI=TRI(ind,:);

if FlagPlot
    figure
    triplot(TRI,imag(Z),real(Z),'k')
    axis ij;axis image
    xlabel('y')
    ylabel('x')
    title('Mesh')
    drawnow
end
