function  [Mesh]=SimpleMesh2(d,Boundary,FlagPlot)

% given a polygonal domain of vertices za (in the complex plane),
% a T3 mesh of approximative element size d is produced.  
% nodes are given as a vector of complex coordinates Z
% NNodeIn is the number of inner nodes Z(1:NNodeIn)
% NNodeEdge is the number of nodes on the boundary Z(NNodeIn+1:end)
% TRI gives the element list (triplets of node numbers)
% iff FlagPlot=1 figures are shown

NBoundary=size(Boundary,2);
for ii=1:NBoundary
    isOuter(ii)=strcmp(Boundary(ii).Type,'Outer');
end
indOuter=find(isOuter);
ZOuter=Boundary(indOuter).Z;

randn('state',sum(100*clock))
x0=min(real(ZOuter));x1=max(real(ZOuter));
y0=min(imag(ZOuter));y1=max(imag(ZOuter));
x=[x0:d:x1];y=[y0:d:y1];
[Y,X]=meshgrid(y,x);
Y(1:2:end,:)=Y(1:2:end,:)+d/2;
% X(:)=X(:)+randn(length(X(:)),1)*d/10;
% Y(:)=Y(:)+randn(length(Y(:)),1)*d/10;
Zcenter=mean(ZOuter);
dis=mean(abs(ZOuter-Zcenter));
zzz=1-d/dis;
IN = inpolygon(X,Y,real(Zcenter+zzz*(ZOuter-Zcenter)),imag(Zcenter+zzz*(ZOuter-Zcenter)));
Z=X+i*Y;
ind=find(IN==1);
Z=Z(ind);
NNodeIn=length(Z);


ij=1;
ik=NNodeIn+1;
for ii=1:NBoundary
    if ii~=indOuter
        zb=Boundary(ii).Z;
        iclose=Boundary(ii).Close;
        Mesh.Close(ij)=iclose;
        Mesh.Type{ij}=Boundary(ii).Type;
        if iclose
            zb=[zb(:);zb(1)];
        end
        ZEdgei=SamplePolyBoundary(d,zb,0).';
        if iclose
            ZEdgei=ZEdgei(1:end-1);
        end
        Z=[Z;ZEdgei];
        NNodeEdge(ij)=length(ZEdgei);
        INodeEdge(ij)=ik;
        ik=ik+NNodeEdge(ij);
        ij=ij+1;
    end
end
iclose=Boundary(indOuter).Close;
Mesh.Close(ij)=iclose;
Mesh.RefPoly=Boundary(indOuter).RefPoly;
Mesh.Type{ij}=Boundary(indOuter).Type;
if iclose
    zb=[ZOuter(:);ZOuter(1)];
end
ZEdgei=SamplePolyBoundary(d,zb,0).';
if iclose
    ZEdgei=ZEdgei(1:end-1);
end
Z=[Z;ZEdgei];
NNodeEdge(ij)=length(ZEdgei);
INodeEdge(ij)=ik;
ZEdge=Z(INodeEdge(1):end);
for ii=1:NNodeIn
    dist=min(abs(ZEdge(:)-Z(ii)));
    ivalid(ii)=(dist>(1.1*d));
end
ninvalid=sum(ivalid==0);
ind=find(ivalid==1);
Z=Z(ind);
Z=[Z;ZEdge];
INodeEdge=INodeEdge-ninvalid;
NNodeTot=length(Z);
TRI=delaunay(real(Z),imag(Z));
NNodeTot=length(Z);
NNodeIn=NNodeIn-ninvalid;

for iter=1:10
    for jter=1:10
        Fz=zeros(NNodeIn,1);
        for ii=1:size(TRI,1)
            ind=TRI(ii,:);
            for jj=1:3
                if ind(jj)<=NNodeIn
                    ki=ind(jj);
                    kj=ind(mod(jj,3)+1);
                    kk=ind(mod(jj+1,3)+1);
                    Fz(ki)=Fz(ki)+(Z(kj)-Z(ki))+(Z(kk)-Z(ki));
                end
            end
        end
        Z(1:NNodeIn)=Z(1:NNodeIn)+Fz(1:NNodeIn)/10;
    end
    TRI=delaunay(real(Z),imag(Z));
end

% Loop for non convex boundaries
for ii=1:size(TRI,1)
    iinode=TRI(ii,:);
    Zcenter=mean(Z(iinode(:)));
    valid(ii)=inpolygon(real(Zcenter),imag(Zcenter),real(ZOuter),imag(ZOuter));
end
ind=find(valid);
TRI=TRI(ind,:);

if FlagPlot
    figure
    triplot(TRI,imag(Z),real(Z),'k')
    axis ij;axis image
    xlabel('$y$','Interpreter','latex')
    ylabel('$x$','Interpreter','latex')
    title('Mesh')
    for ii=1:NBoundary
        line(imag(Z(INodeEdge(ii):INodeEdge(ii)+NNodeEdge(ii)-1)),...
             real(Z(INodeEdge(ii):INodeEdge(ii)+NNodeEdge(ii)-1)),...
             'Color','r','Marker','.');
    end
    drawnow
end

Mesh.TRI=TRI;
Mesh.Znode=Z;
Mesh.NNodeIn=NNodeIn;
Mesh.NNodeEdge=NNodeEdge;
Mesh.INodeEdge=INodeEdge;
Mesh.NBoundary=NBoundary;
Mesh.NNodeTot=sum(NNodeEdge)+NNodeIn;
Mesh.RefMesh=fix(clock);
Mesh.CharLength=d;
Mesh.Nelem=size(Mesh.TRI,1);
Mesh.BParent(1:NBoundary)=0;

