function  [Mesh]=SimpleMesh_v2(Region,Boundary,FlagPlot)

% second version of SimpleMesh
% New features: 
%      multiple boundaries (outer, inner_closed, inner_x_outer,...)
%          Boundary.Type{i}=outer,inner,
%  TBD                                  crack_inner,crack_x (crossing
%                                                            another boundary)
%          Boundary.Z(:,ipoly)=node coordinates (1:nZ(ipoly),ipoly)
%          Boundary.NNode(ipoly)=number of nodes
%   TBD by nodes not boundaries       Boundary.CL=FreeNeumann or Dirichlet
%                      Boundary(1) is by default the one which contains the
%                      entire field
%    multiple regions with different charact. length
%    TBD      Region(i).Z=polygon defining the region
%                       note that Region(1) is the entire meshed domain
%                       Regions are swept through according to their number
%                       larger label regions overlap smaller label regions
%          Region(i).CharLength=d
%
% given a polygonal domain of vertices za (in the complex plane),
% a T3 mesh of approximative element size d is produced.  
% nodes are given as a vector of complex coordinates Z
% NNodeIn is the number of inner nodes Z(1:NNodeIn)
% NNodeEdge is the number of nodes on the boundary Z(NNodeIn+1:end)
% TRI gives the element list (triplets of node numbers)
% iff FlagPlot=1 figures are shown

randn('state',sum(100*clock))
za=Boundary.Z(1:Boundary.NNode(1),1);
d=Region(1).CharLength;
d2=d^2;

x0=min(real(za));x1=max(real(za));
y0=min(imag(za));y1=max(imag(za));
x=[x0:d:x1];y=[y0:d:y1];
[Y,X]=meshgrid(y,x);
Y(1:2:end,:)=Y(1:2:end,:)+d/2;
IN = inpolygon(Y,X,imag(za),real(za));
ind=find(IN==1);
Z=X(ind)+i*Y(ind);
% for ipoly=2:Boundary.NPoly
%     %kill Z node inside polygon is type is inner
%     if strcmp(Boundary.Type{ipoly},'inner')
%         zb=Boundary.Z(1:Boundary.NNode(ipoly),ipoly);
%         ind=find(~inpolygon(imag(Z),real(Z),...
%             imag(zb),real(zb)));
%         Z=Z(ind);
%     end
% end
NNodeIn=length(Z);
status=ones(NNodeIn,1);

ik=0;
za=Boundary.Z(1:Boundary.NNode(1),1);
Zedge=SamplePolyBoundary(d,za,0);
Zedge=Zedge(1:end-1);
NNodeEdge(1)=length(Zedge);
INodeEdge(1,1)=ik+1;
INodeEdge(2,1)=ik+NNodeEdge(1);
ik=INodeEdge(2,1);
for inode=1:NNodeEdge(1)
    dist=abs(Z(:)-Zedge(inode));
    status=status&(dist>d*.75);
end
Zb=[Zedge(1:NNodeEdge(1),1)];
for ipoly=2:Boundary.NPoly
    zb=Boundary.Z(1:Boundary.NNode(ipoly),ipoly);
    % kill Z node inside polygon is type is inner
    if strcmp(Boundary.Type{ipoly},'inner')
        ind=find(inpolygon(imag(Z),real(Z),...
        imag(zb),real(zb)));
        status(ind)=0;
    end
    Zedge2=SamplePolyBoundary(d,zb,0);
    NNodeEdge(ipoly)=length(Zedge2);
    Zedge(1:NNodeEdge(ipoly),ipoly)=Zedge2;
    INodeEdge(1,ipoly)=ik+1;
    INodeEdge(2,ipoly)=ik+NNodeEdge(ipoly);
    ik=INodeEdge(2,ipoly);
    for inode=1:NNodeEdge(ipoly)
        dist=abs(Z(:)-Zedge(inode));
        status=status&(dist>d*.75);
    end
    Zb=[Zb;Zedge(1:NNodeEdge(ipoly),ipoly)]; 
end

Z=Z(status);
NNodeIn=length(Z);
INodeEdge=INodeEdge+NNodeIn;
Z=[Z;Zb];

TRI=delaunay(imag(Z),real(Z));
Npttot=length(Z);

for ii=1:size(TRI,1)
    i1=TRI(ii,1);i2=TRI(ii,2);i3=TRI(ii,3);
    z2=Z(i2)-Z(i1);z3=Z(i3)-Z(i1);
    S(ii)=abs(real(z2)*imag(z3)-real(z3)*imag(z2));
end
ind=find(S>0.01);
TRI=TRI(ind,:);

for iter=1:10
    for jter=1:10
        Fz=zeros(NNodeIn,1);
        for ii=1:size(TRI,1)
            ind=TRI(ii,:);
            for jj=1:3
                if ind(jj)<=NNodeIn
                    ki=ind(jj);
                    kj=ind(mod(jj,3)+1);
                    kk=ind(mod(jj+1,3)+1);
                    Fz(ki)=Fz(ki)+(Z(kj)-Z(ki))+(Z(kk)-Z(ki));
                end
            end
        end
        Z(1:NNodeIn)=Z(1:NNodeIn)+Fz(1:NNodeIn)/10;
    end
    TRI=delaunay(real(Z),imag(Z));
end

for ii=1:size(TRI,1)
    i1=TRI(ii,1);i2=TRI(ii,2);i3=TRI(ii,3);
    z2=Z(i2)-Z(i1);z3=Z(i3)-Z(i1);
    S(ii)=abs(real(z2)*imag(z3)-real(z3)*imag(z2));
end
ind=find(S>0.01);
TRI=TRI(ind,:);

%Loop for non convex outer boundary
za=Boundary.Z(1:Boundary.NNode(1),1);
for ii=1:size(TRI,1)
    iinode=TRI(ii,:);
    Zcenter=mean(Z(iinode(:)));
    valid(ii)=inpolygon(real(Zcenter),imag(Zcenter),real(za),imag(za));
end
ind=find(valid);
TRI=TRI(ind,:);
clear valid
for ipoly=2:Boundary.NPoly
    zb=Boundary.Z(1:Boundary.NNode(ipoly),ipoly);
    for ii=1:size(TRI,1)
        iinode=TRI(ii,:);
        Zcenter=mean(Z(iinode(:)));
        valid(ii)=~inpolygon(imag(Zcenter),real(Zcenter),imag(zb),real(zb));
    end
    ind=find(valid);
    TRI=TRI(ind,:);
    clear valid zb
end


if FlagPlot
    figure
    triplot(TRI,imag(Z),real(Z),'k')
    axis ij;axis image
    xlabel('y')
    ylabel('x')
    title('Mesh')
    drawnow
end

Mesh.TRI=TRI;
Mesh.Znode=Z;
Mesh.NNodeIn=NNodeIn;
Mesh.NNodeEdge=NNodeEdge;
Mesh.INodeEdge=INodeEdge;
Mesh.NBoundary=Boundary.NPoly;
Mesh.NNodeTot=sum(NNodeEdge)+NNodeIn;
Mesh.RefMesh=fix(clock);
Mesh.CharLength=Region(1).CharLength;
Mesh.Nelem=size(Mesh.TRI,1);
Mesh.BParent(1:Mesh.NBoundary)=0;
Mesh.Type=Boundary.Type;
