%Program TestMesh2Image

close all
clear all
set(0,'DefaultAxesFontName','Times','DefaultAxesFontSize',24,'DefaultTextInterpreter','latex');

% Reference: cercle unit reprsent fonctionnellement r-1=0
% Dforme: polygone

m=9;      % number of edges for def polygon
dmesh=3;% distance between points in the mesh generation
randn('state',sum(100*clock));

% Deformee polygone de sommet (complexes) za
t=([0:m]/m-0.5)*pi*2;
za=1.05*exp(i*t);
za=za+(randn(size(za))+i*+randn(size(za)))*.05;
za(end)=za(1);
za=za*100+150+150*i;
ROI=[1 1 300 300];

[TRI,Znode,Nptin,Nptedge]=SimpleMesh(dmesh,za,1);
xlim([ROI(1) ROI(3)]);
ylim([ROI(2) ROI(4)]);
Npttot=Nptin+Nptedge;
Nelem=size(TRI,1); 
Ndof=2*Npttot;


% clear Znode
% Znode(1)=50*(1+i);
% Znode(2)=50*(5+i);
% Znode(3)=50*(1+5*i);
% Znode(4)=50*(5+5*i);
% TRI=[1 2 3; 2 3 4];
% Nelem=2;
% Npttot=4;
% Nptin=0;
% Nptedge=4;

[T,W1,W2]=Pixel2Tri(TRI,Znode,ROI);


% figure
% imagesc(T)
% axis image

% % Test pixel interpolation of grid data
% x=[1:300];
% y=[1:300];
% [Y,X]=meshgrid(y,x);
% testfunc=((X-150).^2+(Y-150).^2).^.25;
% testmesh=interp2(Y,X,testfunc,imag(Znode(:)),real(Znode(:)));
% 
% testproj=interpmesh(ROI,T,W1,W2,TRI,testmesh);
% 
% figure
% imagesc(testproj)
% colormap(hot)
% axis image
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

E=1;
nu=0.2;
mu=E/2/(1+nu);
lambda=(E/(1-2*nu)-2*mu)/3;
kappa=2*mu/3+lambda;


j1=[];
j2=[];
kel=[];
for ii=1:Nelem
    ia=TRI(ii,1);
    ib=TRI(ii,2);
    ic=TRI(ii,3);
    inddof=repmat([2*ia-1;2*ia;2*ib-1;2*ib;2*ic-1;2*ic],[1 6]);
    j1=cat(1,j1,inddof(:));
    inddof=inddof';
    j2=cat(1,j2,inddof(:));
    Z=[Znode(ia),Znode(ib),Znode(ic)];
    [K,M,P]=StiffElem(Z,mu,lambda);
    kel=cat(1,kel,K(:));
end
clear K;
K=sparse(j1,j2,kel);


% Impose Dirichlet condition
Ztrans=0;%2.821454+i*1.6654;
Zrot=0;
Zhomo=0;  %eps= 1  1 0
Zeps1=0;  %eps= 1 -1 0
Zeps2=1;  %eps= 0  0 1
Zd=Znode(Nptin+1:end);
Zcenter=mean(Zd);
Udz=Ztrans+(Zhomo+i*Zrot)*(Zd(:)-Zcenter)+(Zeps1+i*Zeps2)*conj(Zd(:)-Zcenter);

Udz=5.23444/1000*(Zd(:)-Zcenter).^2;

Ud=reshape([real(Udz(:))';imag(Udz(:))'],[2*Nptedge 1]);

F1=-K(1:2*Nptin,2*Nptin+1:2*Npttot)*Ud(:);
Ui=K(1:2*Nptin,1:2*Nptin)\F1;
U=[Ui;Ud];

Uxim=InterpMeshNode(ROI,T,W1,W2,TRI,U(1:2:end));
Uyim=InterpMeshNode(ROI,T,W1,W2,TRI,U(2:2:end));

figure
imagesc(Uxim)
colormap(hot)
axis image
colorbar
xlabel('$y$')
ylabel('$x$')
title('$U_x$')

figure
imagesc(Uyim)
colormap(hot)
axis image
colorbar
xlabel('$y$')
ylabel('$x$')
title('$U_y$')

for ii=1:Nelem
    ia=TRI(ii,1);
    ib=TRI(ii,2);
    ic=TRI(ii,3);
    Z=[Znode(ia),Znode(ib),Znode(ic)];
    [Kel,M,P]=StiffElem(Z,mu,lambda);
    Uel=[U(2*ia-1);U(2*ia);U(2*ib-1);U(2*ib);U(2*ic-1);U(2*ic)];
    eps=M*Uel;
    sig=P*Uel;
    epsxx(ii)=eps(1);
    epsyy(ii)=eps(2);
    epsxy(ii)=eps(3)/sqrt(2);
    sigxx(ii)=sig(1);
    sigyy(ii)=sig(2);
    sigxy(ii)=sig(3)/sqrt(2);
end

Epsxx=InterpMeshElem(ROI,T,epsxx);
Epsyy=InterpMeshElem(ROI,T,epsyy);
Epsxy=InterpMeshElem(ROI,T,epsxy);
Sigxx=InterpMeshElem(ROI,T,sigxx);
Sigyy=InterpMeshElem(ROI,T,sigyy);
Sigxy=InterpMeshElem(ROI,T,sigxy);

figure
imagesc(Epsxx)
colormap(hot)
axis image
colorbar
xlabel('$y$')
ylabel('$x$')
title('$\varepsilon_{xx}$')

figure
imagesc(Epsxy)
colormap(hot)
axis image
colorbar
xlabel('$y$')
ylabel('$x$')
title('$\varepsilon_{xy}$')

figure
imagesc(Epsyy)
colormap(hot)
axis image
colorbar
xlabel('$y$')
ylabel('$x$')
title('$\varepsilon_{yy}$')

figure
imagesc(Sigxx)
colormap(hot)
axis image
colorbar
xlabel('$y$')
ylabel('$x$')
title('$\sigma_{xx}$')

figure
imagesc(Sigxy)
colormap(hot)
axis image
colorbar
xlabel('$y$')
ylabel('$x$')
title('$\sigma_{xy}$')

figure
imagesc(Sigyy)
colormap(hot)
axis image
colorbar
xlabel('$y$')
ylabel('$x$')
title('$\sigma_{yy}$')
    
    
    
    
    
    